"""OpenCTI Connector module.

This module defines the connector types and the main OpenCTIConnector class
used to register and configure connectors with the OpenCTI platform.
"""

from enum import Enum


class ConnectorType(Enum):
    """Enumeration of OpenCTI connector types.

    Each connector type defines a specific data flow pattern:

    - EXTERNAL_IMPORT: Imports data from remote sources into OpenCTI as STIX2
    - INTERNAL_IMPORT_FILE: Converts files from OpenCTI file system to STIX2
    - INTERNAL_ENRICHMENT: Enriches existing STIX2 data with additional information
    - INTERNAL_ANALYSIS: Analyzes files or STIX2 data and produces file output
    - INTERNAL_EXPORT_FILE: Exports STIX2 data to files in OpenCTI file system
    - STREAM: Reads the event stream and performs custom actions

    Scope definition varies by type:
        - EXTERNAL_IMPORT: None (imports everything)
        - INTERNAL_IMPORT_FILE: MIME types to support (e.g., application/json)
        - INTERNAL_ENRICHMENT: Entity types to support (e.g., Report, Hash)
        - INTERNAL_EXPORT_FILE: MIME types to generate (e.g., application/pdf)
    """

    EXTERNAL_IMPORT = "EXTERNAL_IMPORT"
    INTERNAL_IMPORT_FILE = "INTERNAL_IMPORT_FILE"
    INTERNAL_ENRICHMENT = "INTERNAL_ENRICHMENT"
    INTERNAL_ANALYSIS = "INTERNAL_ANALYSIS"
    INTERNAL_EXPORT_FILE = "INTERNAL_EXPORT_FILE"
    STREAM = "STREAM"


class OpenCTIConnector:
    """Main class for OpenCTI connector registration and configuration.

    This class represents a connector instance that can be registered with
    the OpenCTI platform. It holds all configuration parameters needed for
    the connector to operate.

    :param connector_id: Unique identifier for the connector (valid UUID4)
    :type connector_id: str
    :param connector_name: Human-readable name for the connector
    :type connector_name: str
    :param connector_type: Type of connector (see :class:`ConnectorType`)
    :type connector_type: str
    :param scope: Connector scope as a comma-separated string (e.g., "Report,Indicator")
    :type scope: str
    :param auto: Whether the connector runs automatically on matching entities
    :type auto: bool
    :param only_contextual: Whether the connector only processes contextual data
    :type only_contextual: bool
    :param playbook_compatible: Whether the connector can be used in playbooks
    :type playbook_compatible: bool
    :param auto_update: Whether to automatically update existing entities
    :type auto_update: bool
    :param enrichment_resolution: Strategy for resolving enrichment conflicts
    :type enrichment_resolution: str
    :param listen_callback_uri: Optional callback URI for API-based listening
    :type listen_callback_uri: str or None

    :raises ValueError: If the connector type is not a valid ConnectorType value

    Example:
        >>> connector = OpenCTIConnector(
        ...     connector_id="550e8400-e29b-41d4-a716-446655440000",
        ...     connector_name="My Connector",
        ...     connector_type="EXTERNAL_IMPORT",
        ...     scope="Report,Indicator",
        ...     auto=False,
        ...     only_contextual=False,
        ...     playbook_compatible=True,
        ...     auto_update=False,
        ...     enrichment_resolution="none"
        ... )
    """

    def __init__(
        self,
        connector_id: str,
        connector_name: str,
        connector_type: str,
        scope: str,
        auto: bool,
        only_contextual: bool,
        playbook_compatible: bool,
        auto_update: bool,
        enrichment_resolution: str,
        listen_callback_uri=None,
    ):
        """Initialize the OpenCTIConnector instance.

        :param connector_id: Unique identifier for the connector (valid UUID4)
        :type connector_id: str
        :param connector_name: Human-readable name for the connector
        :type connector_name: str
        :param connector_type: Type of connector (see :class:`ConnectorType`)
        :type connector_type: str
        :param scope: Connector scope as a comma-separated string
        :type scope: str
        :param auto: Whether the connector runs automatically
        :type auto: bool
        :param only_contextual: Whether to process only contextual data
        :type only_contextual: bool
        :param playbook_compatible: Whether the connector works with playbooks
        :type playbook_compatible: bool
        :param auto_update: Whether to auto-update existing entities
        :type auto_update: bool
        :param enrichment_resolution: Enrichment conflict resolution strategy
        :type enrichment_resolution: str
        :param listen_callback_uri: Optional callback URI for API listening
        :type listen_callback_uri: str or None

        :raises ValueError: If connector_type is not a valid ConnectorType
        """
        self.id = connector_id
        self.name = connector_name
        self.type = ConnectorType(connector_type)
        if scope:
            self.scope = scope.split(",")
        else:
            self.scope = []
        self.auto = auto
        self.auto_update = auto_update
        self.enrichment_resolution = enrichment_resolution
        self.only_contextual = only_contextual
        self.playbook_compatible = playbook_compatible
        self.listen_callback_uri = listen_callback_uri

    def to_input(self) -> dict:
        """Convert connector configuration to API input format.

        Generates a dictionary structure suitable for use in GraphQL API
        queries to register or update the connector.

        :return: Dictionary containing connector data wrapped in an "input" key
        :rtype: dict

        Example:
            >>> connector.to_input()
            {'input': {'id': '...', 'name': 'My Connector', ...}}
        """
        return {
            "input": {
                "id": self.id,
                "name": self.name,
                "type": self.type.name,
                "scope": self.scope,
                "auto": self.auto,
                "auto_update": self.auto_update,
                "enrichment_resolution": self.enrichment_resolution,
                "only_contextual": self.only_contextual,
                "playbook_compatible": self.playbook_compatible,
                "listen_callback_uri": self.listen_callback_uri,
            }
        }



import json


class StixObjectOrStixRelationship:
    """Main StixObjectOrStixRelationship class for OpenCTI

    Manages generic STIX objects and relationships in the OpenCTI platform.

    :param opencti: instance of :py:class:`~pycti.api.opencti_api_client.OpenCTIApiClient`
    :type opencti: OpenCTIApiClient
    """

    def __init__(self, opencti):
        """Initialize the StixObjectOrStixRelationship instance.

        :param opencti: OpenCTI API client instance
        :type opencti: OpenCTIApiClient
        """
        self.opencti = opencti
        self.properties = """
            ... on StixObject {
                id
                standard_id
                entity_type
                parent_types
                spec_version
                created_at
                updated_at
            }
            ... on StixDomainObject {
                createdBy {
                    ... on Identity {
                        id
                        standard_id
                        entity_type
                        parent_types
                        spec_version
                        identity_class
                        name
                        description
                        roles
                        contact_information
                        x_opencti_aliases
                        created
                        modified
                        objectLabel {
                            id
                            value
                            color
                        }
                    }
                    ... on Organization {
                        x_opencti_organization_type
                        x_opencti_reliability
                    }
                    ... on Individual {
                        x_opencti_firstname
                        x_opencti_lastname
                    }
                }
                objectOrganization {
                    id
                    standard_id
                    name
                }
                objectMarking {
                    id
                    standard_id
                    entity_type
                    definition_type
                    definition
                    created
                    modified
                    x_opencti_order
                    x_opencti_color
                }
                objectLabel {
                    id
                    value
                    color
                }
                externalReferences {
                    edges {
                        node {
                            id
                            standard_id
                            entity_type
                            source_name
                            description
                            url
                            hash
                            external_id
                            created
                            modified
                        }
                    }
                }
                revoked
                confidence
                created
                modified
            }
            ... on AttackPattern {
                name
                description
                aliases
                x_mitre_platforms
                x_mitre_permissions_required
                x_mitre_detection
                x_mitre_id
                killChainPhases {
                  id
                  standard_id
                  entity_type
                  kill_chain_name
                  phase_name
                  x_opencti_order
                  created
                  modified
                }
            }
            ... on Campaign {
                name
                description
                aliases
                first_seen
                last_seen
                objective
            }
            ... on Note {
                attribute_abstract
                content
                authors
                note_types
                likelihood
            }
            ... on ObservedData {
                first_observed
                last_observed
                number_observed
            }
            ... on Opinion {
                explanation
                authors
                opinion
            }
            ... on Report {
                name
                description
                report_types
                published
            }
            ... on CourseOfAction {
                name
                description
                x_opencti_aliases
            }
            ... on Individual {
                name
                description
                contact_information
                x_opencti_aliases
                x_opencti_firstname
                x_opencti_lastname
            }
            ... on Organization {
                name
                description
                contact_information
                x_opencti_aliases
                x_opencti_organization_type
                x_opencti_reliability
            }
            ... on Sector {
                name
                description
                contact_information
                x_opencti_aliases
            }
            ... on System {
                name
                description
                contact_information
                x_opencti_aliases
            }
            ... on Indicator {
                pattern_type
                pattern_version
                pattern
                name
                description
                indicator_types
                valid_from
                valid_until
                x_opencti_score
                x_opencti_detection
                x_opencti_main_observable_type
            }
            ... on Infrastructure {
                name
                description
                aliases
                infrastructure_types
                first_seen
                last_seen
            }
            ... on IntrusionSet {
                name
                description
                aliases
                first_seen
                last_seen
                goals
                resource_level
                primary_motivation
                secondary_motivations
            }
            ... on City {
                name
                description
                latitude
                longitude
                precision
                x_opencti_aliases
            }
            ... on Country {
                name
                description
                latitude
                longitude
                precision
                x_opencti_aliases
            }
            ... on  Region {
                name
                description
                latitude
                longitude
                precision
                x_opencti_aliases
            }
            ... on Position {
                name
                description
                latitude
                longitude
                precision
                x_opencti_aliases
                street_address
                postal_code
            }
            ... on Malware {
                name
                description
                aliases
                malware_types
                is_family
                first_seen
                last_seen
                architecture_execution_envs
                implementation_languages
                capabilities
                killChainPhases {
                  id
                  standard_id
                  entity_type
                  kill_chain_name
                  phase_name
                  x_opencti_order
                  created
                  modified
                }
            }
            ... on ThreatActor {
                name
                description
                aliases
                threat_actor_types
                first_seen
                last_seen
                roles
                goals
                sophistication
                resource_level
                primary_motivation
                secondary_motivations
                personal_motivations
            }
            ... on Tool {
                name
                description
                aliases
                tool_types
                tool_version
                killChainPhases {
                  id
                  standard_id
                  entity_type
                  kill_chain_name
                  phase_name
                  x_opencti_order
                  created
                  modified
                }
            }
            ... on Vulnerability {
                name
                description
                x_opencti_aliases
                x_opencti_cvss_vector_string
                x_opencti_cvss_base_score
                x_opencti_cvss_base_severity
                x_opencti_cvss_attack_vector
                x_opencti_cvss_attack_complexity
                x_opencti_cvss_privileges_required
                x_opencti_cvss_user_interaction
                x_opencti_cvss_scope
                x_opencti_cvss_confidentiality_impact
                x_opencti_cvss_integrity_impact
                x_opencti_cvss_availability_impact
                x_opencti_cvss_exploit_code_maturity
                x_opencti_cvss_remediation_level
                x_opencti_cvss_report_confidence
                x_opencti_cvss_temporal_score
                x_opencti_cvss_v2_vector_string
                x_opencti_cvss_v2_base_score
                x_opencti_cvss_v2_access_vector
                x_opencti_cvss_v2_access_complexity
                x_opencti_cvss_v2_authentication
                x_opencti_cvss_v2_confidentiality_impact
                x_opencti_cvss_v2_integrity_impact
                x_opencti_cvss_v2_availability_impact
                x_opencti_cvss_v2_exploitability
                x_opencti_cvss_v2_remediation_level
                x_opencti_cvss_v2_report_confidence
                x_opencti_cvss_v2_temporal_score
                x_opencti_cvss_v4_vector_string
                x_opencti_cvss_v4_base_score
                x_opencti_cvss_v4_base_severity
                x_opencti_cvss_v4_attack_vector
                x_opencti_cvss_v4_attack_complexity
                x_opencti_cvss_v4_attack_requirements
                x_opencti_cvss_v4_privileges_required
                x_opencti_cvss_v4_user_interaction
                x_opencti_cvss_v4_confidentiality_impact_v
                x_opencti_cvss_v4_confidentiality_impact_s
                x_opencti_cvss_v4_integrity_impact_v
                x_opencti_cvss_v4_integrity_impact_s
                x_opencti_cvss_v4_availability_impact_v
                x_opencti_cvss_v4_availability_impact_s
                x_opencti_cvss_v4_exploit_maturity
                x_opencti_cwe
                x_opencti_cisa_kev
                x_opencti_epss_score
                x_opencti_epss_percentile
                x_opencti_score
            }
            ... on Incident {
                name
                description
                aliases
                first_seen
                last_seen
                objective
            }
            ... on Event {
                name
                description
              }
             ... on Channel {
                name
                description
              }
             ... on Narrative {
                name
                description
              }
             ... on Language {
                name
            }
            ... on DataComponent {
                name
                description
            }
            ... on DataSource {
                name
                description
            }
            ... on Case {
                name
            }
            ... on StixCyberObservable {
                observable_value
            }
            ... on StixCoreRelationship {
                id
                standard_id
                entity_type
                parent_types
                createdBy {
                    ... on Identity {
                        id
                        standard_id
                        entity_type
                        parent_types
                        name
                        x_opencti_aliases
                        description
                        created
                        modified
                    }
                    ... on Organization {
                        x_opencti_organization_type
                        x_opencti_reliability
                    }
                    ... on Individual {
                        x_opencti_firstname
                        x_opencti_lastname
                    }
                }
                objectMarking {
                    id
                    standard_id
                    entity_type
                    definition_type
                    definition
                    created
                    modified
                    x_opencti_order
                    x_opencti_color
                }
                objectLabel {
                    id
                    value
                    color
                }
                externalReferences {
                    edges {
                        node {
                            id
                            standard_id
                            entity_type
                            source_name
                            description
                            url
                            hash
                            external_id
                            created
                            modified
                        }
                    }
                }
                revoked
                confidence
                created
                modified
                description
                start_time
                stop_time
            }
            ... on StixSightingRelationship {
                id
                standard_id
                entity_type
                parent_types
                createdBy {
                    ... on Identity {
                        id
                        standard_id
                        entity_type
                        parent_types
                        name
                        x_opencti_aliases
                        description
                        created
                        modified
                    }
                    ... on Organization {
                        x_opencti_organization_type
                        x_opencti_reliability
                    }
                    ... on Individual {
                        x_opencti_firstname
                        x_opencti_lastname
                    }
                }
                objectMarking {
                    id
                    standard_id
                    entity_type
                    definition_type
                    definition
                    created
                    modified
                    x_opencti_order
                    x_opencti_color
                }
                objectLabel {
                    id
                    value
                    color
                }
                externalReferences {
                    edges {
                        node {
                            id
                            standard_id
                            entity_type
                            source_name
                            description
                            url
                            hash
                            external_id
                            created
                            modified
                        }
                    }
                }
                confidence
                created
                modified
                description
                attribute_count
                x_opencti_negative
                first_seen
                last_seen
            }
        """

    def read(self, **kwargs):
        """Read a StixObjectOrStixRelationship object.

        :param id: the id of the StixObjectOrStixRelationship
        :type id: str
        :param customAttributes: custom attributes to return
        :type customAttributes: str
        :param filters: the filters to apply
        :type filters: dict
        :return: StixObjectOrStixRelationship object
        :rtype: dict or None
        """
        id = kwargs.get("id", None)
        custom_attributes = kwargs.get("customAttributes", None)
        filters = kwargs.get("filters", None)
        if id is not None:
            self.opencti.app_logger.info(
                "Reading StixObjectOrStixRelationship", {"id": id}
            )
            query = (
                """
                    query StixObjectOrStixRelationship($id: String!) {
                        stixObjectOrStixRelationship(id: $id) {
                            """
                + (
                    custom_attributes
                    if custom_attributes is not None
                    else self.properties
                )
                + """
                    }
                }
             """
            )
            result = self.opencti.query(query, {"id": id})
            return self.opencti.process_multiple_fields(
                result["data"]["stixObjectOrStixRelationship"]
            )
        elif filters is not None:
            result = self.list(filters=filters)
            if len(result) > 0:
                return result[0]
            else:
                return None
        else:
            self.opencti.app_logger.error("Missing parameters: id")
            return None

    def list(self, **kwargs):
        """List StixObjectOrStixRelationship objects.

        :param filters: the filters to apply
        :type filters: dict
        :param search: the search keyword
        :type search: str
        :param first: return the first n rows from the after ID (or the beginning if not set)
        :type first: int
        :param after: ID of the first row for pagination
        :type after: str
        :param getAll: whether to retrieve all results
        :type getAll: bool
        :param with_pagination: whether to include pagination info
        :type with_pagination: bool
        :param customAttributes: custom attributes to return
        :type customAttributes: str
        :return: List of StixObjectOrStixRelationship objects
        :rtype: list
        """
        filters = kwargs.get("filters", None)
        search = kwargs.get("search", None)
        first = kwargs.get("first", 100)
        after = kwargs.get("after", None)
        get_all = kwargs.get("getAll", False)
        with_pagination = kwargs.get("with_pagination", False)
        custom_attributes = kwargs.get("customAttributes", None)

        self.opencti.app_logger.info(
            "Listing StixObjectOrStixRelationships with filters",
            {"filters": json.dumps(filters)},
        )
        query = (
            """
                        query StixObjectOrStixRelationships($filters: FilterGroup, $search: String, $first: Int, $after: ID) {
                            stixObjectOrStixRelationships(filters: $filters, search: $search, first: $first, after: $after) {
                                edges {
                                    node {
                                        """
            + (custom_attributes if custom_attributes is not None else self.properties)
            + """
                                }
                            }
                            pageInfo {
                                startCursor
                                endCursor
                                hasNextPage
                                hasPreviousPage
                                globalCount
                            }
                        }
                    }
                """
        )
        variables = {
            "filters": filters,
            "search": search,
            "first": first,
            "after": after,
        }
        result = self.opencti.query(
            query,
            variables,
        )

        if get_all:
            final_data = []
            data = self.opencti.process_multiple(
                result["data"]["stixObjectOrStixRelationships"]
            )
            final_data = final_data + data
            while result["data"]["stixObjectOrStixRelationships"]["pageInfo"][
                "hasNextPage"
            ]:
                after = result["data"]["stixObjectOrStixRelationships"]["pageInfo"][
                    "endCursor"
                ]
                self.opencti.app_logger.debug(
                    "Listing stixObjectOrStixRelationships", {"after": after}
                )
                after_variables = {**variables, **{"after": after}}
                result = self.opencti.query(query, after_variables)
                data = self.opencti.process_multiple(
                    result["data"]["stixObjectOrStixRelationships"]
                )
                final_data = final_data + data
            return final_data
        else:
            return self.opencti.process_multiple(
                result["data"]["stixObjectOrStixRelationships"], with_pagination
            )



class OpenCTIApiPlaybook:
    """OpenCTI Playbook API class.

    Manages playbook operations.

    :param api: instance of :py:class:`~pycti.api.opencti_api_client.OpenCTIApiClient`
    :type api: OpenCTIApiClient
    """

    def __init__(self, api):
        """Initialize the OpenCTIApiPlaybook instance.

        :param api: OpenCTI API client instance
        :type api: OpenCTIApiClient
        """
        self.api = api

    def playbook_step_execution(self, playbook: dict, bundle: str):
        """Execute a playbook step.

        :param playbook: the playbook configuration dict containing execution_id, event_id,
            execution_start, playbook_id, data_instance_id, step_id, previous_step_id,
            and previous_bundle
        :type playbook: dict
        :param bundle: the STIX bundle to process
        :type bundle: str
        :return: None
        :rtype: None
        """
        self.api.app_logger.info(
            "Executing playbook step",
            {
                "playbook_id": playbook["playbook_id"],
                "step_id": playbook["step_id"],
                "data_instance_id": playbook["data_instance_id"],
            },
        )
        query = """
            mutation PlaybookStepExecution($execution_id: ID!, $event_id: ID!, $execution_start: DateTime!, $data_instance_id: ID!, $playbook_id: ID!, $previous_step_id: ID!, $step_id: ID!, $previous_bundle: String!, $bundle: String!) {
                playbookStepExecution(execution_id: $execution_id, event_id: $event_id, execution_start: $execution_start, data_instance_id: $data_instance_id, playbook_id: $playbook_id, previous_step_id: $previous_step_id, step_id: $step_id, previous_bundle: $previous_bundle, bundle: $bundle)
            }
           """
        self.api.query(
            query,
            {
                "execution_id": playbook["execution_id"],
                "event_id": playbook["event_id"],
                "execution_start": playbook["execution_start"],
                "playbook_id": playbook["playbook_id"],
                "data_instance_id": playbook["data_instance_id"],
                "step_id": playbook["step_id"],
                "previous_step_id": playbook["previous_step_id"],
                "previous_bundle": playbook["previous_bundle"],
                "bundle": bundle,
            },
        )

    def delete(self, **kwargs):
        """Delete a playbook.

        :param id: the playbook id
        :type id: str
        :return: None
        :rtype: None
        """
        playbook_id = kwargs.get("id", None)
        if playbook_id is not None:
            query = """
                mutation PlaybookDelete($id: ID!) {
                    playbookDelete(id: $id)
                }
               """
            self.api.query(
                query,
                {
                    "id": playbook_id,
                },
            )
        else:
            self.api.app_logger.error(
                "[opencti_playbook] Cannot delete playbook, missing parameter: id"
            )
            return None



"""OpenCTI Metric Handler module.

This module provides Prometheus metrics support for OpenCTI connectors,
allowing monitoring of connector performance and health.
"""

from typing import Type, Union

from prometheus_client import Counter, Enum, start_http_server


class OpenCTIMetricHandler:
    """Handler for Prometheus metrics in OpenCTI connectors.

    This class manages Prometheus metrics for monitoring connector behavior,
    including bundle sends, records processed, run counts, API pings, and errors.

    When activated, it starts an HTTP server to expose metrics for scraping
    by Prometheus or compatible monitoring systems.

    :param connector_logger: Logger instance for the connector
    :type connector_logger: logging.Logger
    :param activated: Whether to enable metrics collection and exposure
    :type activated: bool
    :param namespace: Prometheus metrics namespace prefix
    :type namespace: str
    :param subsystem: Prometheus metrics subsystem prefix
    :type subsystem: str
    :param port: Port number for the Prometheus HTTP server
    :type port: int

    Example:
        >>> handler = OpenCTIMetricHandler(
        ...     connector_logger=logger,
        ...     activated=True,
        ...     namespace="opencti",
        ...     subsystem="connector",
        ...     port=9095
        ... )
        >>> handler.inc("bundle_send")
        >>> handler.state("running")
    """

    def __init__(
        self,
        connector_logger,
        activated: bool = False,
        namespace: str = "",
        subsystem: str = "",
        port: int = 9095,
    ):
        """Initialize the OpenCTIMetricHandler instance.

        :param connector_logger: Logger instance for the connector
        :type connector_logger: logging.Logger
        :param activated: Whether to enable metrics (default: False)
        :type activated: bool
        :param namespace: Prometheus metrics namespace prefix (default: "")
        :type namespace: str
        :param subsystem: Prometheus metrics subsystem prefix (default: "")
        :type subsystem: str
        :param port: Port for Prometheus HTTP server (default: 9095)
        :type port: int
        """
        self.activated = activated
        self.connector_logger = connector_logger
        if self.activated:
            self.connector_logger.info("Exposing metrics on port", {"port": port})
            start_http_server(port)
            self._metrics = {
                "bundle_send": Counter(
                    "bundles_sent_total",
                    "Number of bundles sent",
                    namespace=namespace,
                    subsystem=subsystem,
                ),
                "record_send": Counter(
                    "records_sent_total",
                    "Number of records (objects per bundle) sent",
                    namespace=namespace,
                    subsystem=subsystem,
                ),
                "run_count": Counter(
                    "runs_total",
                    "Number of runs",
                    namespace=namespace,
                    subsystem=subsystem,
                ),
                "ping_api_count": Counter(
                    "ping_api_total",
                    "Number of pings to the API",
                    namespace=namespace,
                    subsystem=subsystem,
                ),
                "ping_api_error": Counter(
                    "ping_api_errors_total",
                    "Number of errors when pinging the API",
                    namespace=namespace,
                    subsystem=subsystem,
                ),
                "error_count": Counter(
                    "errors_total",
                    "Number of errors",
                    namespace=namespace,
                    subsystem=subsystem,
                ),
                "client_error_count": Counter(
                    "client_errors_total",
                    "Number of client errors",
                    namespace=namespace,
                    subsystem=subsystem,
                ),
                "state": Enum(
                    "state",
                    "State of connector",
                    states=["idle", "running", "stopped"],
                    namespace=namespace,
                    subsystem=subsystem,
                ),
            }

    def _metric_exists(
        self, name: str, expected_type: Union[Type[Counter], Type[Enum]]
    ) -> bool:
        """Check if a metric exists and has the expected type.

        Validates that the named metric is registered and matches the expected
        Prometheus metric type. Logs an error if validation fails.

        :param name: Name of the metric to validate
        :type name: str
        :param expected_type: Expected Prometheus metric type (Counter or Enum)
        :type expected_type: Union[Type[Counter], Type[Enum]]

        :return: True if metric exists and has correct type, False otherwise
        :rtype: bool
        """
        if name not in self._metrics:
            self.connector_logger.error("Metric does not exist.", {"name": name})
            return False
        if not isinstance(self._metrics[name], expected_type):
            self.connector_logger.error(
                "Metric not of expected type",
                {"name": name, "expected_type": expected_type},
            )
            return False
        return True

    def inc(self, name: str, n: int = 1) -> None:
        """Increment a counter metric by a specified amount.

        Increments the named counter metric. If metrics are not activated
        or the metric does not exist, this method does nothing.

        Available counter metrics:
            - bundle_send: Number of bundles sent
            - record_send: Number of records sent
            - run_count: Number of connector runs
            - ping_api_count: Number of API pings
            - ping_api_error: Number of API ping errors
            - error_count: Total number of errors
            - client_error_count: Number of client errors

        :param name: Name of the counter metric to increment
        :type name: str
        :param n: Amount to increment the counter by (default: 1)
        :type n: int

        Example:
            >>> handler.inc("bundle_send")
            >>> handler.inc("record_send", 10)
        """
        if self.activated:
            if self._metric_exists(name, Counter):
                self._metrics[name].inc(n)

    def state(self, state: str, name: str = "state") -> None:
        """Set the state of an Enum metric.

        Updates the named Enum metric to the specified state value.
        If metrics are not activated or the metric does not exist,
        this method does nothing.

        Available states for the default "state" metric:
            - idle: Connector is idle
            - running: Connector is running
            - stopped: Connector is stopped

        :param state: State value to set (must be a valid state for the metric)
        :type state: str
        :param name: Name of the Enum metric to update (default: "state")
        :type name: str

        Example:
            >>> handler.state("running")
            >>> handler.state("idle", "state")
        """
        if self.activated:
            if self._metric_exists(name, Enum):
                self._metrics[name].state(state)



class OpenCTIApiDraft:
    """OpenCTI Draft API class.

    Manages draft workspace operations.

    :param api: instance of :py:class:`~pycti.api.opencti_api_client.OpenCTIApiClient`
    :type api: OpenCTIApiClient
    """

    def __init__(self, api):
        """Initialize the OpenCTIApiDraft instance.

        :param api: OpenCTI API client instance
        :type api: OpenCTIApiClient
        """
        self.api = api

    def delete(self, **kwargs):
        """Delete a draft workspace.

        :param id: the draft workspace id
        :type id: str
        :return: None
        :rtype: None
        """
        draft_id = kwargs.get("id", None)
        if draft_id is None:
            self.api.app_logger.error(
                "[opencti_draft] Cannot delete draft workspace, missing parameter: id"
            )
            return None
        query = """
            mutation DraftWorkspaceDelete($id: ID!) {
                draftWorkspaceDelete(id: $id)
            }
           """
        self.api.query(
            query,
            {
                "id": draft_id,
            },
        )



# coding: utf-8

import json
import uuid

from stix2.canonicalization.Canonicalize import canonicalize


class SecurityCoverage:
    """Main SecurityCoverage class for OpenCTI

    Manages security coverage entities in the OpenCTI platform.

    :param opencti: instance of :py:class:`~pycti.api.opencti_api_client.OpenCTIApiClient`
    :type opencti: OpenCTIApiClient
    """

    def __init__(self, opencti):
        """Initialize the SecurityCoverage instance.

        :param opencti: OpenCTI API client instance
        :type opencti: OpenCTIApiClient
        """
        self.opencti = opencti
        self.properties = """
            id
            standard_id
            entity_type
            parent_types
            spec_version
            created_at
            updated_at
            external_uri
            objectCovered {
                __typename
                ... on StixCoreObject {
                  id
                }
            }
            objectMarking {
                id
                standard_id
                entity_type
                definition_type
                definition
                created
                modified
                x_opencti_order
                x_opencti_color
            }
        """

    @staticmethod
    def generate_id(covered_ref):
        """Generate a STIX ID for a Security Coverage.

        :param covered_ref: The reference to the covered object
        :type covered_ref: str
        :return: STIX ID for the security coverage
        :rtype: str
        """
        data = {"covered_ref": covered_ref.lower().strip()}
        data = canonicalize(data, utf8=False)
        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
        return "security-coverage--" + id

    @staticmethod
    def generate_id_from_data(data):
        """Generate a STIX ID from security coverage data.

        :param data: Dictionary containing 'covered_ref' key
        :type data: dict
        :return: STIX ID for the security coverage
        :rtype: str
        """
        return SecurityCoverage.generate_id(data["covered_ref"])

    def list(self, **kwargs):
        """List SecurityCoverage objects.

        :param filters: the filters to apply
        :param search: the search keyword
        :param first: return the first n rows from the after ID (or the beginning if not set)
        :param after: ID of the first row for pagination
        :return: List of SecurityCoverage objects
        :rtype: list
        """
        filters = kwargs.get("filters", None)
        search = kwargs.get("search", None)
        first = kwargs.get("first", 100)
        after = kwargs.get("after", None)
        order_by = kwargs.get("orderBy", None)
        order_mode = kwargs.get("orderMode", None)
        custom_attributes = kwargs.get("customAttributes", None)
        get_all = kwargs.get("getAll", False)
        with_pagination = kwargs.get("withPagination", False)

        self.opencti.app_logger.info(
            "Listing SecurityCoverage with filters", {"filters": json.dumps(filters)}
        )
        query = (
            """
                query SecurityCoverage($filters: FilterGroup, $search: String, $first: Int, $after: ID, $orderBy: SecurityCoverageOrdering, $orderMode: OrderingMode) {
                    securityCoverages(filters: $filters, search: $search, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
                        edges {
                            node {
                                """
            + (custom_attributes if custom_attributes is not None else self.properties)
            + """
                        }
                    }
                    pageInfo {
                        startCursor
                        endCursor
                        hasNextPage
                        hasPreviousPage
                        globalCount
                    }
                }
            }
        """
        )
        result = self.opencti.query(
            query,
            {
                "filters": filters,
                "search": search,
                "first": first,
                "after": after,
                "orderBy": order_by,
                "orderMode": order_mode,
            },
        )

        if get_all:
            final_data = []
            data = self.opencti.process_multiple(result["data"]["securityCoverages"])
            final_data = final_data + data
            while result["data"]["securityCoverages"]["pageInfo"]["hasNextPage"]:
                after = result["data"]["securityCoverages"]["pageInfo"]["endCursor"]
                self.opencti.app_logger.info(
                    "Listing SecurityCoverage", {"after": after}
                )
                result = self.opencti.query(
                    query,
                    {
                        "filters": filters,
                        "search": search,
                        "first": first,
                        "after": after,
                        "orderBy": order_by,
                        "orderMode": order_mode,
                    },
                )
                data = self.opencti.process_multiple(
                    result["data"]["securityCoverages"]
                )
                final_data = final_data + data
            return final_data
        else:
            return self.opencti.process_multiple(
                result["data"]["securityCoverages"], with_pagination
            )

    def read(self, **kwargs):
        """Read a SecurityCoverage object.

        :param id: the id of the SecurityCoverage
        :type id: str
        :param filters: the filters to apply if no id provided
        :type filters: dict
        :param customAttributes: custom attributes to return
        :type customAttributes: str
        :return: SecurityCoverage object
        :rtype: dict or None
        """
        id = kwargs.get("id", None)
        filters = kwargs.get("filters", None)
        custom_attributes = kwargs.get("customAttributes", None)
        if id is not None:
            self.opencti.app_logger.info("Reading SecurityCoverage", {"id": id})
            query = (
                """
                    query SecurityCoverage($id: String!) {
                        securityCoverage(id: $id) {
                            """
                + (
                    custom_attributes
                    if custom_attributes is not None
                    else self.properties
                )
                + """
                    }
                }
             """
            )
            result = self.opencti.query(query, {"id": id})
            return self.opencti.process_multiple_fields(
                result["data"]["securityCoverage"]
            )
        elif filters is not None:
            result = self.list(filters=filters)
            if len(result) > 0:
                return result[0]
            else:
                return None
        else:
            self.opencti.app_logger.error(
                "[opencti_security_coverage] Missing parameters: id or filters"
            )
            return None

    def create(self, **kwargs):
        """Create a Security coverage object.

        :param name: the name of the Security Coverage (required)
        :type name: str
        :param objectCovered: the covered object ID (required)
        :type objectCovered: str
        :param stix_id: (optional) the STIX ID
        :type stix_id: str
        :param description: (optional) description
        :type description: str
        :param createdBy: (optional) the author ID
        :type createdBy: str
        :param objectMarking: (optional) list of marking definition IDs
        :type objectMarking: list
        :param objectLabel: (optional) list of label IDs
        :type objectLabel: list
        :param externalReferences: (optional) list of external reference IDs
        :type externalReferences: list
        :param external_uri: (optional) external URI
        :type external_uri: str
        :param coverage_last_result: (optional) last result date
        :type coverage_last_result: str
        :param coverage_valid_from: (optional) valid from date
        :type coverage_valid_from: str
        :param coverage_valid_to: (optional) valid to date
        :type coverage_valid_to: str
        :param coverage_information: (optional) coverage information
        :type coverage_information: list
        :param auto_enrichment_disable: (optional) disable auto enrichment
        :type auto_enrichment_disable: bool
        :param periodicity: (optional) periodicity
        :type periodicity: str
        :param duration: (optional) duration
        :type duration: str
        :param type_affinity: (optional) type affinity
        :type type_affinity: str
        :param platforms_affinity: (optional) platforms affinity
        :type platforms_affinity: list
        :param files: (optional) list of File objects to attach
        :type files: list
        :param filesMarkings: (optional) list of lists of marking definition IDs for each file
        :type filesMarkings: list
        :return: Security Coverage object
        :rtype: dict or None
        """
        stix_id = kwargs.get("stix_id", None)
        name = kwargs.get("name", None)
        description = kwargs.get("description", None)
        created_by = kwargs.get("createdBy", None)
        object_marking = kwargs.get("objectMarking", None)
        object_label = kwargs.get("objectLabel", None)
        object_covered = kwargs.get("objectCovered", None)
        external_references = kwargs.get("externalReferences", None)
        external_uri = kwargs.get("external_uri", None)
        coverage_last_result = kwargs.get("coverage_last_result", None)
        coverage_valid_from = kwargs.get("coverage_valid_from", None)
        coverage_valid_to = kwargs.get("coverage_valid_to", None)
        coverage_information = kwargs.get("coverage_information", None)
        auto_enrichment_disable = kwargs.get("auto_enrichment_disable", None)
        periodicity = kwargs.get("periodicity", None)
        duration = kwargs.get("duration", None)
        type_affinity = kwargs.get("type_affinity", None)
        platforms_affinity = kwargs.get("platforms_affinity", None)
        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
        files = kwargs.get("files", None)
        files_markings = kwargs.get("filesMarkings", None)
        no_trigger_import = kwargs.get("noTriggerImport", None)
        embedded = kwargs.get("embedded", None)

        if name is not None and object_covered is not None:
            self.opencti.app_logger.info("Creating Security Coverage", {"name": name})
            query = """
                mutation SecurityCoverageAdd($input: SecurityCoverageAddInput!) {
                    securityCoverageAdd(input: $input) {
                        id
                        standard_id
                        entity_type
                        parent_types
                    }
                }
            """
            result = self.opencti.query(
                query,
                {
                    "input": {
                        "stix_id": stix_id,
                        "name": name,
                        "description": description,
                        "createdBy": created_by,
                        "objectMarking": object_marking,
                        "objectLabel": object_label,
                        "objectCovered": object_covered,
                        "external_uri": external_uri,
                        "externalReferences": external_references,
                        "coverage_last_result": coverage_last_result,
                        "coverage_valid_from": coverage_valid_from,
                        "coverage_valid_to": coverage_valid_to,
                        "coverage_information": coverage_information,
                        "auto_enrichment_disable": auto_enrichment_disable,
                        "periodicity": periodicity,
                        "duration": duration,
                        "type_affinity": type_affinity,
                        "platforms_affinity": platforms_affinity,
                        "x_opencti_stix_ids": x_opencti_stix_ids,
                        "files": files,
                        "filesMarkings": files_markings,
                        "noTriggerImport": no_trigger_import,
                        "embedded": embedded,
                    }
                },
            )
            return self.opencti.process_multiple_fields(
                result["data"]["securityCoverageAdd"]
            )
        else:
            self.opencti.app_logger.error(
                "[opencti_security_coverage] "
                "Missing parameters: name or object_covered"
            )
            return None

    def import_from_stix2(self, **kwargs):
        """Import a Security coverage from a STIX2 object.

        :param stixObject: the STIX2 Security coverage object
        :type stixObject: dict
        :param extras: extra parameters including created_by_id, object_marking_ids, etc.
        :type extras: dict
        :param update: whether to update if the entity already exists
        :type update: bool
        :return: Security coverage object
        :rtype: dict or None
        """
        stix_object = kwargs.get("stixObject", None)
        extras = kwargs.get("extras", {})
        if stix_object is not None:
            # Search in extensions
            if "x_opencti_stix_ids" not in stix_object:
                stix_object["x_opencti_stix_ids"] = (
                    self.opencti.get_attribute_in_extension("stix_ids", stix_object)
                )
            if "x_opencti_granted_refs" not in stix_object:
                stix_object["x_opencti_granted_refs"] = (
                    self.opencti.get_attribute_in_extension("granted_refs", stix_object)
                )

            raw_coverages = stix_object["coverage"] if "coverage" in stix_object else []
            coverage_information = list(
                map(
                    lambda cov: {
                        "coverage_name": cov["name"],
                        "coverage_score": cov["score"],
                    },
                    raw_coverages,
                )
            )

            return self.create(
                stix_id=stix_object["id"],
                name=stix_object["name"],
                external_uri=(
                    stix_object["external_uri"]
                    if "external_uri" in stix_object
                    else None
                ),
                auto_enrichment_disable=(
                    stix_object["auto_enrichment_disable"]
                    if "auto_enrichment_disable" in stix_object
                    else False
                ),
                periodicity=(
                    stix_object["periodicity"] if "periodicity" in stix_object else None
                ),
                duration=(
                    stix_object["duration"] if "duration" in stix_object else None
                ),
                type_affinity=(
                    stix_object["type_affinity"]
                    if "type_affinity" in stix_object
                    else None
                ),
                platforms_affinity=(
                    stix_object["platforms_affinity"]
                    if "platforms_affinity" in stix_object
                    else None
                ),
                coverage_last_result=(
                    stix_object["last_result"] if "last_result" in stix_object else None
                ),
                coverage_valid_from=(
                    stix_object["valid_from"] if "valid_from" in stix_object else None
                ),
                coverage_valid_to=(
                    stix_object["valid_to"] if "valid_to" in stix_object else None
                ),
                coverage_information=coverage_information,
                description=(
                    self.opencti.stix2.convert_markdown(stix_object["description"])
                    if "description" in stix_object
                    else None
                ),
                createdBy=(
                    extras["created_by_id"] if "created_by_id" in extras else None
                ),
                objectMarking=(
                    extras["object_marking_ids"]
                    if "object_marking_ids" in extras
                    else None
                ),
                objectLabel=(
                    extras["object_label_ids"] if "object_label_ids" in extras else None
                ),
                objectCovered=(
                    stix_object["covered_ref"] if "covered_ref" in stix_object else None
                ),
                externalReferences=(
                    extras["external_references_ids"]
                    if "external_references_ids" in extras
                    else None
                ),
                x_opencti_stix_ids=(
                    stix_object["x_opencti_stix_ids"]
                    if "x_opencti_stix_ids" in stix_object
                    else None
                ),
                files=extras.get("files"),
                filesMarkings=extras.get("filesMarkings"),
                noTriggerImport=extras.get("noTriggerImport", None),
                embedded=extras.get("embedded", None),
            )
        else:
            self.opencti.app_logger.error(
                "[opencti_security_coverage] Missing parameters: stixObject"
            )
            return None
